using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Xml;
using System.Xml.Serialization;
using Java.Lang;
using String = System.String;

namespace Master.Core
{
    public class Pedal: IPedal
    {

        private readonly List<Preset> _presetsList;
        public string Name { get; set; }
        public string Id { get; set; }
        public string Version { get; set; }
        public SerializableDictionary<string, Option> Options { get; set; }

        private readonly string _documents = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
        private readonly IController _controller;
        public Pedal() { }
        public Pedal(String pedalString, IController controller)
        {
            var parameters = pedalString.Split(',');
            var options = new SerializableDictionary<string, Option>();
            for (var i = 0; i<parameters.Length; i++)
            {
                switch (i)
                {
                    case 0:
                        Name = parameters[i];
                        break;
                    case 1:
                        Id = parameters[i];
                        break;
                    case 2:
                        Version = parameters[i];
                        break;
                    default:
                    {
                        var option = CreateOption(parameters[i]);
                        options.Add(option.Name, option);
                    }
                        break;
                }
                
            }
            Options = options;
             _controller = controller;
            _presetsList = ReadPresetsFromFile();
        }

        private static Option CreateOption(string optionString)
        {
            var parameters = optionString.Split(':');
            return new Option(parameters[0], int.Parse(parameters[1]), int.Parse(parameters[2]),
                int.Parse(parameters[3]));;
        }


        private List<Preset> ReadPresetsFromFile()
        {
            
            var filename = Path.Combine(_documents, "preset_" + Name);
            if (!File.Exists(filename)) return new List<Preset>();
            using (var streamReader = new StreamReader(filename))
            {
                var serializer = new XmlSerializer(typeof(List<Preset>));
                return serializer.Deserialize(streamReader) as List<Preset>;
            }
        }

        private bool WritePresetsToFile()
        {
            try
            {
                var filename = Path.Combine(_documents, "preset_" + Name);
                //var document = new XmlDocument();
                //document.Save(new StreamWriter(filename, false));
                //new StreamWriter(filename, false);
                using (var writer = new StreamWriter(filename))
                {
                    var serializer = new XmlSerializer(_presetsList.GetType());
                    serializer.Serialize(writer, _presetsList);
                    writer.Flush();

                }
            }
            catch (IOException e)
            {
                Console.WriteLine(e.Message);
                return false;
            }
           
            return true;
        }
        public void SetOption(string name, int value)
        {
            if (value > Options[name].Max) return;
            if (value < Options[name].Min) return;
            _controller.UpdateOption(Id, name, value);
            Options[name].Value = value;
        }
        public void IncreaseOption(string optionName)
        {
            SetOption(optionName, Options[optionName].Value+1);
        }

        public void DecreaseOption(string optionName)
        {
            SetOption(optionName, Options[optionName].Value-1);
        }

        public void SaveToPresets(string presetName)
        {
            var copy = new SerializableDictionary<string, Option>();
            foreach (var e in Options)
            {
                copy.Add(e.Key, e.Value.Clone());
            }

            var preset = new Preset {Name = presetName, Options = copy};
            _presetsList.Add(preset);
            var res = WritePresetsToFile();
          }

        public void ApplyPreset(string presetId)
        {
            var preset = _presetsList.FirstOrDefault(p => p.GetId().Equals(presetId));
            if (preset == null) return;
            foreach (var option in preset.Options.Values)
            {
                SetOption(option.Name, option.Value);
             }
        }

        public bool RemoveFromPresets(string presetId)
        {
            foreach (var preset in _presetsList.Where(preset => preset.GetId().Equals(presetId)))
            {
                _presetsList.Remove(preset);
                WritePresetsToFile();
                return true;
            }
            return false;
        }

        public List<Preset> GetPresets()
        {
            return _presetsList;
        }

        public override string ToString()
        {
            return Id + " : " + Name + " version: " + Version;
        }
    }
    public class Preset

    {
        public string Name { get; set; }
        private readonly string _id = Guid.NewGuid().ToString("n");
        public SerializableDictionary<string, Option> Options { get; set; }
        public string GetId()
        {
            return _id;
        }

        public override string ToString()
        {
            return Name;
        }
    }


    public class Option
{
    public Option()
    {

    }

    public Option(string name, int value, int min, int max)
    {
        Max = max;
        Value = value;
        Name = name;
        Min = min;
    }

        public Option Clone()
        {
            return (Option)this.MemberwiseClone();
        }
    public string Name { get; set; }
    public int Value { get; set; }
    public int Max { get; set; }
    public int Min { get; set; }

}


}